@@ -96,7 +96,7 @@ from lbmpy.maxwellian_equilibrium import (
 )
 from lbmpy.moments import (
     MOMENT_SYMBOLS, exponent_tuple_sort_key, exponents_to_polynomial_representations,
-    extract_monomials, moment_sort_key,
+    extract_monomials, moment_sort_key, moment_matrix,
     monomial_to_polynomial_transformation_matrix, set_up_shift_matrix)
 
 FORCE_SYMBOLS = sp.symbols("F_x, F_y, F_z")
@@ -131,6 +131,7 @@ class AbstractForceModel(abc.ABC):
         # central moment space
         self.has_moment_space_forcing = hasattr(self, "moment_space_forcing")
         self.has_central_moment_space_forcing = hasattr(self, "central_moment_space_forcing")
+        self.has_symmetric_central_moment_forcing = hasattr(self, "symmetric_central_moment_forcing")
 
     def __call__(self, lb_method):
         r"""
@@ -201,6 +202,13 @@ class Simple(AbstractForceModel):
         moments = (lb_method.moment_matrix * sp.Matrix(self(lb_method))).expand()
         return lb_method.shift_matrix * moments
 
+    def symmetric_central_moment_forcing(self, lb_method, central_moments):
+        u = lb_method.first_order_equilibrium_moment_symbols
+        cm_matrix = moment_matrix(central_moments, lb_method.stencil, shift_velocity=u)
+        before = sp.Matrix([0] * lb_method.stencil.Q)
+        after = cm_matrix @ sp.Matrix(self(lb_method))
+        return before, after
+
 
 class Luo(AbstractForceModel):
     r"""Force model by Luo :cite:`luo1993lattice`.
@@ -232,6 +240,13 @@ class Luo(AbstractForceModel):
         moments = lb_method.moment_matrix * self(lb_method)
         return (lb_method.shift_matrix * moments).expand()
 
+    def symmetric_central_moment_forcing(self, lb_method, central_moments):
+        u = lb_method.first_order_equilibrium_moment_symbols
+        cm_matrix = moment_matrix(central_moments, lb_method.stencil, shift_velocity=u)
+        before = sp.Matrix([0] * lb_method.stencil.Q)
+        after = (cm_matrix @ sp.Matrix(self(lb_method))).expand()
+        return before, after
+
 
 class Guo(AbstractForceModel):
     r"""
@@ -269,6 +284,12 @@ class Guo(AbstractForceModel):
 
         return central_moments
 
+    def symmetric_central_moment_forcing(self, lb_method, central_moments):
+        luo = Luo(self.symbolic_force_vector)
+        _, force_cms = luo.symmetric_central_moment_forcing(lb_method, central_moments)
+        force_cms = sp.Rational(1, 2) * force_cms
+        return force_cms, force_cms
+
     def equilibrium_velocity_shift(self, density):
         return default_velocity_shift(density, self.symbolic_force_vector)
 
@@ -324,14 +345,14 @@ class He(AbstractForceModel):
 
         return sp.Matrix(result)
 
-    def continuous_force_raw_moments(self, lb_method):
+    def continuous_force_raw_moments(self, lb_method, moments=None):
         rho = lb_method.zeroth_order_equilibrium_moment_symbol
         u = lb_method.first_order_equilibrium_moment_symbols
         dim = lb_method.dim
         c_s_sq = sp.Rational(1, 3)
         force = sp.Matrix(self.symbolic_force_vector)
 
-        moment_polynomials = lb_method.moments
+        moment_polynomials = lb_method.moments if moments is None else moments
         moment_exponents = sorted(extract_monomials(moment_polynomials), key=exponent_tuple_sort_key)
         moment_monomials = exponents_to_polynomial_representations(moment_exponents)
         extended_monomials = set()
@@ -354,10 +375,12 @@ class He(AbstractForceModel):
         polynomial_force_moments = mono_to_poly_matrix * sp.Matrix(monomial_force_moments)
         return polynomial_force_moments
 
-    def continuous_force_central_moments(self, lb_method):
-        raw_moments = self.continuous_force_raw_moments(lb_method)
+    def continuous_force_central_moments(self, lb_method, moments=None):
+        if moments is None:
+            moments = lb_method.moments
+        raw_moments = self.continuous_force_raw_moments(lb_method, moments=moments)
         u = lb_method.first_order_equilibrium_moment_symbols
-        shift_matrix = set_up_shift_matrix(lb_method.moments, lb_method.stencil, velocity_symbols=u)
+        shift_matrix = set_up_shift_matrix(moments, lb_method.stencil, velocity_symbols=u)
         return (shift_matrix * raw_moments).expand()
 
     def __call__(self, lb_method):
@@ -384,6 +407,11 @@ class He(AbstractForceModel):
         central_moments = (correction_factor * central_moments).expand()
         return central_moments
 
+    def symmetric_central_moment_forcing(self, lb_method, central_moments):
+        central_moments = exponents_to_polynomial_representations(central_moments)
+        force_cms = sp.Rational(1, 2) * self.continuous_force_central_moments(lb_method, moments=central_moments)
+        return force_cms, force_cms
+
     def equilibrium_velocity_shift(self, density):
         return default_velocity_shift(density, self.symbolic_force_vector)
 
